use std::{cell::RefCell, ops::Range};
mod decoder;
mod encoder;
mod slice2d;
pub use decoder::*;
pub use encoder::*;
use slice2d::*;

pub struct ImageData {
    pub data: RefCell<[u64; 64]>,
}

impl Default for ImageData {
    fn default() -> Self {
        Self {
            data: RefCell::new([0u64; 64]),
        }
    }
}

impl<'a> ImageData {
    pub fn get(&self, index: (usize, usize)) -> bool {
        ((self.data.borrow_mut()[index.1] >> index.0) & !1) > 0
    }
    pub fn set(&self, value: bool, index: (usize, usize)) {
        self.data.borrow_mut()[index.1] = if value {
            self.data.borrow_mut()[index.1] | (1 << index.0)
        } else {
            self.data.borrow_mut()[index.1] & !(1 << index.0)
        }
    }
    pub fn get_slice2d(&'a self, region: (Range<usize>, Range<usize>)) -> Slice2D<'a> {
        Slice2D {
            buf: self,
            h: region.0,
            v: region.1,
        }
    }
}

pub enum Element {
    ControlCode {
        data: u128,
    },
    Glyph {
        courtesy_space: bool,
        image_data: Box<ImageData>,
    },
}
