use std::ops::Range;

use crate::ImageData;

pub struct Slice2D<'a> {
    pub buf: &'a ImageData,
    pub h: Range<usize>,
    pub v: Range<usize>,
}

impl<'a> Slice2D<'a> {
    // Relative coordinates
    pub fn get(&self, index: (usize, usize)) -> bool {
        self.buf
            .get((index.0 + self.h.start, index.1 + self.v.start))
    }
    pub fn set(&self, value: bool, index: (usize, usize)) {
        self.buf
            .set(value, (index.0 + self.h.start, index.1 + self.v.start));
    }
    pub fn get_all(&self) -> Option<bool> {
        let initial = self.get((0, 0));
        for v in 0..self.v.clone().len() {
            for h in 0..self.h.clone().len() {
                if self.get((h, v)) != initial {
                    return None;
                }
            }
        }
        Some(initial)
    }
    pub fn set_all(&self, value: bool) {
        for v in 0..self.v.clone().len() {
            for h in 0..self.h.clone().len() {
                self.set(value, (h, v))
            }
        }
    }
    pub fn get_slice2d(&'a self, region: (Range<usize>, Range<usize>)) -> Slice2D<'a> {
        Slice2D {
            buf: self.buf,
            h: region.0.start + self.h.start..region.0.end + self.h.start,
            v: region.1.start + self.v.start..region.1.end + self.v.start,
        }
    }
    pub fn len(&self) -> (usize, usize) {
        (self.h.len(), self.v.len())
    }
}
